import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Write a description of class Projectile here.
 * 
 * @author (your name) 
 * @version (a version number or a date)
 */
public class Projectile extends Actor
{
    private double exactX;
    private double exactY;
    
    private Player thrower;
    
    private final double velX;
    private double velY; // positive is upwards
        
    private int rotation = 0;
    private GreenfootImage original;
    
    public Projectile(Player thrower, double vx, double vy)
    {
        velX = vx;
        velY = vy;
        this.thrower = thrower;
        
        original = getImage();
    }   
    

    /**
     * Act - do whatever the Projectile wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {
        fly();        
        spin();
    }    
    
    private void fly()
    {
        velY -= ProjectileWorld.GRAVITY;
        moveTo(exactX + velX, exactY - velY);
    }
    
    private void spin()
    {
        rotation += 8;
        GreenfootImage cur = new GreenfootImage(original);
        cur.rotate(rotation);
        setImage(cur);
    }
    
    private void moveTo(double x, double y)
    {
        ProjectileWorld world = (ProjectileWorld)getWorld();
        if (world.hitsGround(exactX, exactY, x, y))
        {
            world.removeObject(this);
            Greenfoot.playSound("impact.wav");
            thrower.landed(x, y);
            world.landed();
        }
        else
        {
            setExactLocation(x, y);
            Player p = (Player)getOneIntersectingObject(Player.class);
            if (p != null && p != thrower)
            {
                world.removeObject(this);
                world.hitPlayer(p);
            }
        }
    }
    
    
    public void setLocation(int x, int y)
    {
        exactX = x;
        exactY = y;
        super.setLocation(x, y);
    }
    
    public void setExactLocation(double x, double y) 
    {
        exactX = x;
        exactY = y;
        super.setLocation((int) (x + 0.5), (int) (y + 0.5));
    }
    
    
    
    public static double getImpactTime(double radianAngle, double forceFactor)
    {        
        // X trajectory is: x(t) = t * vx
        // Y trajectory is: y(t) = t * vy - 0.5 * g * t * (t + 1)
        
        // So point of impact is when y(t) = 0 (t > 0)
        // 0 = t * vy - 0.5 * g * t * (t + 1)
        // 0 = t * vy - 0.5 * g * t * t - 0.5 * g * t
        //     a = -0.5 * g
        //     b = vy - 0.5 * g
        //     c = 0
        // t = (0.5 * g - vy +- sqrt( (vy - 0.5 * g)^2  )) / -g
        // t = (0.5 * g - vy +- (vy - 0.5 * g)) / -g
        // t = 0 or -2 * (0.5 * g - vy) / g
        // t = 0 or -1 + 2 * vy / g
        
        double vx = Player.getVelX(radianAngle, forceFactor);
        double vy = Player.getVelY(radianAngle, forceFactor);
        
        double impactTime = 2.0 * (vy / ProjectileWorld.GRAVITY) - 1;
        
        return impactTime;
    }
}
